@@ -19,6 +19,8 @@
 # --------------------------------
 #
 # --------------------------------
+import ast
+import json
 import logging
 import os
 import pprint
@@ -27,6 +29,7 @@ import o7util.file_explorer as o7fe
 import o7util.format as o7f
 import o7util.input as o7i
 import o7util.menu as o7m
+import o7util.terminal as o7t
 from o7util.table import ColumnParam, Table, TableParam
 
 from o7cli.base import Base
@@ -59,6 +62,84 @@ class S3(Base):
         self.objects: list = []
         self.s3_object: dict = {}
 
+    # *************************************************
+    #
+    # *************************************************
+    def verify_ssl_requests_only(self, bucket: str, statements: list) -> bool:
+        """Verify if the bucket policy enforces SSL requests only"""
+        logger.info("verify_ssl_requests_only")
+
+        bucket_arn = f"arn:aws:s3:::{bucket}"
+        bucket_arn_with_wildcard = f"{bucket_arn}/*"
+
+        for statement in statements:
+            # pprint.pprint(statement)
+
+            if statement.get("Effect", "") != "Deny":
+                continue
+
+            resources = statement.get("Resource", [])
+            if not isinstance(resources, list):
+                resources = [resources]
+
+            if (bucket_arn not in resources) or (
+                bucket_arn_with_wildcard not in resources
+            ):
+                continue
+
+            condition = statement.get("Condition", {})
+            if (
+                "Bool" in condition
+                and condition["Bool"].get("aws:SecureTransport") == "false"
+            ):
+                return True
+
+        return False
+
+    # *************************************************
+    #
+    # *************************************************
+    def set_ssl_requests_only(self):
+        """Set the bucket policy to enforce SSL requests only"""
+
+        bucket = self.bucket
+        policies = self.get_bucket_policies(bucket=bucket)
+        policies.append(
+            {
+                "Effect": "Deny",
+                "Principal": "*",
+                "Action": "*",
+                "Resource": [
+                    f"arn:aws:s3:::{bucket}",
+                    f"arn:aws:s3:::{bucket}/*",
+                ],
+                "Condition": {"Bool": {"aws:SecureTransport": "false"}},
+            }
+        )
+
+        policy = {"Version": "2012-10-17", "Statement": policies}
+
+        # https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3.html#S3.Client.put_bucket_policy
+        response = self.client.put_bucket_policy(
+            Bucket=bucket, Policy=json.dumps(policy)
+        )
+
+        return response
+
+    # *************************************************
+    #
+    # *************************************************
+    def get_bucket_policies(self, bucket: str):
+        """Load additional information for a bucket"""
+
+        try:
+            response = self.client.get_bucket_policy(Bucket=bucket)
+        except self.client.exceptions.from_code("NoSuchBucketPolicy"):
+            return []
+
+        policy = response.get("Policy", "")
+        return ast.literal_eval(policy).get("Statement", [])
+
     # *************************************************
     #
     # *************************************************
@@ -74,8 +155,9 @@ class S3(Base):
         self.buckets = resp.get("Buckets", [])
         logger.info(f"LoadBuckets: Number of Bucket found: {len(self.buckets)}")
 
-        # Get region for each bucket
         for bucket in self.buckets:
+
+            # Get region for each bucket
             try:
                 resp = self.client.get_bucket_location(Bucket=bucket["Name"])
             except self.client.exceptions.NoSuchBucket:
@@ -86,7 +168,28 @@ class S3(Base):
             if bucket["Region"] is None:
                 bucket["Region"] = "us-east-1"
 
-        # pprint.pprint(self.buckets)
+            # Policies
+            bucket["Policies"] = self.get_bucket_policies(bucket=bucket["Name"])
+            bucket["IsSSL"] = self.verify_ssl_requests_only(
+                bucket=bucket["Name"], statements=bucket["Policies"]
+            )
+
+        return self
+
+    # *************************************************
+    #
+    # *************************************************
+    def load_bucket(self):
+        """Load information about a bucket"""
+
+        logger.info("load_bucket")
+        bucket = self.bucket
+
+        self.bucket_info = {}
+        self.bucket_info["Policies"] = self.get_bucket_policies(bucket=bucket)
+        self.bucket_info["IsSSL"] = self.verify_ssl_requests_only(
+            bucket=bucket, statements=self.bucket_info["Policies"]
+        )
 
         return self
 
@@ -275,6 +378,8 @@ class S3(Base):
                 ColumnParam(title="Name", type="str", data_col="Name"),
                 ColumnParam(title="Created", type="datetime", data_col="CreationDate"),
                 ColumnParam(title="Region", type="str", data_col="Region"),
+                ColumnParam(title="Region", type="str", data_col="Region"),
+                ColumnParam(title="SSL", type="str", data_col="IsSSL"),
             ]
         )
         print()
@@ -282,6 +387,26 @@ class S3(Base):
 
         return self
 
+    # *************************************************
+    #
+    # *************************************************
+    def display_bucket(self):
+        """Display the list buckets"""
+
+        self.load_bucket()
+
+        print("")
+        print(f"Bucket: {self.bucket}")
+        print("")
+        print("Policies:")
+        pprint.pprint(self.bucket_info["Policies"])
+        print("")
+
+        if self.bucket_info["IsSSL"]:
+            print(o7t.format_normal("SSL Request is enforced"))
+        else:
+            print(o7t.format_alarm("SSL Request is not enforced"))
+
     # *************************************************
     #
     # *************************************************
@@ -348,13 +473,9 @@ class S3(Base):
     # *************************************************
     #
     # *************************************************
-    def menu_folder(self, index: int):
+    def menu_folder(self):
         """S3 Folder menu"""
 
-        if not 0 < index <= len(self.buckets):
-            return self
-
-        self.bucket = self.buckets[index - 1]["Name"]
         self.prefix = ""
         self.key = ""
 
@@ -375,6 +496,42 @@ class S3(Base):
         obj.display_callback = self.display_folder
         obj.loop()
 
+    # *************************************************
+    #
+    # *************************************************
+    def menu_bucket(self, index: int):
+        """S3 Folder menu"""
+
+        if not 0 < index <= len(self.buckets):
+            return self
+
+        self.bucket = self.buckets[index - 1]["Name"]
+        self.prefix = ""
+        self.key = ""
+
+        obj = o7m.Menu(
+            exit_option="b",
+            title=f"S3 Bucket - {self.bucket}",
+            title_extra=self.session_info(),
+            compact=False,
+        )
+        obj.add_option(
+            o7m.Option(
+                key="r", name="Raw", callback=lambda: pprint.pprint(self.objects)
+            )
+        )
+        obj.add_option(o7m.Option(key="f", name="See Files", callback=self.menu_folder))
+        obj.add_option(
+            o7m.Option(
+                key="set-ssl",
+                name="Set SSL Request Policy",
+                callback=self.set_ssl_requests_only,
+            )
+        )
+
+        obj.display_callback = self.display_bucket
+        obj.loop()
+
     # *************************************************
     #
     # *************************************************
@@ -401,7 +558,7 @@ class S3(Base):
                 key="int",
                 name="Details for a bucket",
                 short="Details",
-                callback=self.menu_folder,
+                callback=self.menu_bucket,
             )
         )
 
@@ -423,7 +580,6 @@ def menu(**kwargs):
 #
 # *************************************************
 if __name__ == "__main__":
-
     logging.basicConfig(
         level=logging.INFO, format="[%(levelname)-5.5s] [%(name)s] %(message)s"
     )
@@ -431,11 +587,16 @@ if __name__ == "__main__":
     # the_s3 = S3().menu_buckets()
 
     the_s3 = S3()
-    the_s3.bucket = "my-bucket"
+    # the_s3.bucket = "my-bucket"
     # the_s3.prefix = ""
-    the_s3.key = "my-file.zip"
+    # the_s3.key = "my-file.zip"
     # the_s3.display_folder()
 
-    the_s3.display_object()
+    # the_s3.display_object()
 
     # theS3.MenuBuckets()
+
+    the_s3.load_bucket_extra("appstream2-36fb080bb8-ca-central-1-649732530399")
+    the_s3.load_bucket_extra(
+        "apistack-apibasicservicedownloadbucket31ba7d2a-jg2n5e641xqe"
+    )
